#include "Boxes.h"
#include "Camera.h"

#define BOX_WIDTH_VAR 0
#define BOX_HEIGHT_VAR 1

Boxes::Boxes(int typeID) : Renderable3d(typeID)
{
}

Boxes::~Boxes(void)
{
}

void Boxes::SetVariables()
{
	m_layerVariables[BOX_WIDTH_VAR].Initialize(this, "Box width", 0.2f, GRID_SIZE - 0.1, 0.1f);
	m_layerVariables[BOX_HEIGHT_VAR].Initialize(this, "Box height", 0.2f, 2.0f, 0.2f);
	m_variableCount = 2;
}

void Boxes::BuildCollection()
{
	m_entityCollectionCount = 0;
	Camera* camera = Camera::theCamera;

	//find center of screen
	float center_x = DivFloat(camera->Lookat.x, GRID_SIZE);
	float center_y = DivFloat(camera->Lookat.y, GRID_SIZE);

	//find magnitude of screen
	float screen_magnitude = (camera->Lookat - camera->Position).Magnitude();

	float x_width = DivFloat(screen_magnitude + 2*GRID_SIZE, GRID_SIZE);
	float y_width = DivFloat(screen_magnitude + 2*GRID_SIZE, GRID_SIZE);


	//iterating over space
	float i = -x_width/2 + center_x;
	while(i < x_width/2 + center_x)
	{
		float j = -y_width/2 + center_y;
		while(j < y_width/2 + center_y)
		{
			if(m_entityCollectionCount >= MAX_ENTITIES -1)
				return;

			float boxWidth = m_layerVariables[BOX_WIDTH_VAR].GetValue(i, j, 0.3);
			float boxHeight = m_layerVariables[BOX_HEIGHT_VAR].GetValue(i, j);

			if(boxWidth > 0 && boxHeight > 0)
			{
				m_entityCollection[m_entityCollectionCount].Initialize(i, j, boxWidth, boxWidth, boxHeight);
				m_entityCollectionCount++;
			}
			j += GRID_SIZE;
		}

		i += GRID_SIZE;
	}
}