#include "LayerManager.h"
#include "BaseRenderable.h"
#include "RenderableFactory.h"
#include "Renderable2d.h"
#include "Renderable3d.h"
#include "WindowMapping.h"


LayerManager* LayerManager::m_Manager = NULL;

void LayerManager::Initialize(GLUI* parentGlui)
{
	m_renderMode = DISPLAY_2D;
	m_parentGlui = parentGlui;
	m_panelLayers = parentGlui->add_panel("Layer controls");

#ifdef ENABLE_LAYER_LIST
	m_lbLayers = parentGlui->add_listbox_to_panel(m_panelLayers, "Top Layers: ", 0, -1, LayerManager::event_ChangeSelection);

	//add/remove buttons
	m_btnAddLayer = parentGlui->add_button_to_panel(m_panelLayers, "Add new layer", -1, LayerManager::event_AddLayer);
	m_btnRemoveLayer = parentGlui->add_button_to_panel(m_panelLayers, "Remove layer", -1, LayerManager::event_RemoveLayer);
#else
	//add/remove buttons
	m_btnAddLayer = parentGlui->add_button_to_panel(m_panelLayers, "Change base", -1, LayerManager::event_AddLayer);
#endif

	m_cbInverse = parentGlui->add_checkbox_to_panel(m_panelLayers, "Inverse", 0, -1, LayerManager::event_InverseCheck);

	m_layerVariables = parentGlui->add_panel_to_panel(m_panelLayers, "Layer Variables");
	m_lbVariables = parentGlui->add_listbox_to_panel(m_layerVariables, "Select variable: ", 0, -1, LayerManager::event_VariableChangeSelection);
	
	m_txtVariableValue = parentGlui->add_edittext_to_panel(m_layerVariables, "Value: ");

	m_lbLayerTypes = parentGlui->add_listbox_to_panel(m_layerVariables, "Layer Types: ");
	for(int i = 0; i < RenderableFactory::LayerTypeCount; i++)
	{
		//if(RenderableFactory::TypeInformation[i].type_metatype)
		m_lbLayerTypes->add_item(i, RenderableFactory::TypeInformation[i].type_name);
	}

	m_btnSetInputLayer = parentGlui->add_button_to_panel(m_layerVariables, "Assign input layer", -1, LayerManager::event_SetInputLayer);
	m_btnRemoveInputLayer = parentGlui->add_button_to_panel(m_layerVariables, "Remove input layer", -1, LayerManager::event_RemoveInputLayer);
	m_btnNavigateInto = parentGlui->add_button_to_panel(m_layerVariables, "Edit input layer", -1, LayerManager::event_NavigateInto);

	m_btnNavigateUp = parentGlui->add_button_to_panel(m_panelLayers, "Go to parent", -1, LayerManager::event_NavigateUp);
	
	//save, load
	m_btnLoadXML = parentGlui->add_button("Load from XML", -1, LayerManager::event_loadXML);
	m_btnSaveXML = parentGlui->add_button("Save to XML", -1, LayerManager::event_saveXML);

	SetSelectedLayer();
}

void LayerManager::SetViewportSize(int w, int h)
{
	//should notify all layers about new size.
	m_width = w;
	m_height = h;

	m_mapping2d.SetPhysicalWindow(0, m_width, 0, m_height);
	m_mapping2d.SetWorldWindow(-2.5f, 2.5f, -2.5f, 2.5f);
}

void LayerManager::SetGuiInformation()
{
	m_btnAddLayer->enable();
#ifdef ENABLE_LAYER_LIST
	
	m_btnRemoveLayer->disable();
#endif
	m_btnNavigateUp->disable();

	if(m_currentLayer != NULL)
	{
		//a selected layer
#ifdef ENABLE_LAYER_LIST
		m_btnRemoveLayer->enable();
#endif
		GLUI_Node *node = m_lbVariables->items_list.first_child();
		while(node != NULL)
		{
			int id = ((GLUI_Listbox_Item*)node)->id;
			node = node->next();
			m_lbVariables->delete_item(id);
		}

		for(int i = 0; i < m_currentLayer->m_variableCount; i++)
			m_lbVariables->add_item(i, m_currentLayer->m_layerVariables[i].GetName());

		SetSelectedVariable();
	}
}

void LayerManager::SetVariableOverrideValue()
{
	char txt[100];
	int selectedVariable = m_lbVariables->get_int_val();
	sprintf_s(txt, "%g", m_currentLayer->m_layerVariables[selectedVariable].GetOverrideValue());
	m_txtVariableValue->set_text(txt);
}

void LayerManager::SetSelectedVariable()
{
	m_btnNavigateInto->disable();
	m_btnRemoveInputLayer->disable();
	m_btnSetInputLayer->disable();
	m_btnNavigateUp->disable();

	if(m_currentLayer != NULL)
	{
		SetVariableOverrideValue();

		int selectedVariable = m_lbVariables->get_int_val();

		if(m_currentLayer->m_layerVariables[selectedVariable].GetRenderable() != NULL)
		{
			m_btnNavigateInto->enable();
			m_btnRemoveInputLayer->enable();
		}
		else
		{
			m_btnSetInputLayer->enable();
		}

		if(m_currentLayer->ParentLayer != NULL)
        {
			m_btnNavigateUp->enable();
        }


		//really dumb.  Would be better to use an attribute list
		if(RenderableFactory::TypeInformation[m_currentLayer->m_typeID].type_metatype != RENDERABLE_3D)
		{
			int inversechecked = ((Renderable2d*)m_currentLayer)->m_inverse ? 1 : 0;
			m_cbInverse->set_int_val(inversechecked);
		}

		m_renderMode = RenderableFactory::TypeInformation[m_currentLayer->m_typeID].type_display_mode;

	}

    // Quick fix for rendering problem on OSX while shift-dragging.
	// GLUI seems to be difficult in this area..
	// THANKS, [WODE]!!
    UpdateTextures();
 }

void LayerManager::event_InverseCheck(int value)
{
	m_Manager->InverseChecked();
}

void LayerManager::InverseChecked()
{
	if(m_currentLayer != NULL)
	{
		
		int checked = m_cbInverse->get_int_val();

		//really dumb.  Would be better to use an attribute list
		if(dynamic_cast<Renderable2d*>(m_currentLayer) != NULL)
			((Renderable2d*)m_currentLayer)->m_inverse = (bool)checked;

		m_shouldUpdateLayers = true;
		UpdateTextures();
	}
}

void LayerManager::event_RemoveLayer(int value)
{
	m_Manager->RemoveLayer();
}

void LayerManager::event_ChangeSelection(int value)
{
	m_Manager->SetSelectedLayer();
}

void LayerManager::event_VariableChangeSelection(int value)
{
	m_Manager->SetSelectedVariable();
}

void LayerManager::event_RemoveInputLayer(int value)
{
	m_Manager->RemoveInputLayer();
}

void LayerManager::RemoveInputLayer()
{
	if(m_currentLayer != NULL)
	{
		int selectedIndex = m_lbVariables->get_int_val();
		m_currentLayer->m_layerVariables[selectedIndex].RemoveRenderable();
		SetSelectedVariable();
		m_shouldUpdateLayers = true;
	}
}

void LayerManager::event_SetInputLayer(int value)
{
	m_Manager->SetInputLayer();
}

void LayerManager::SetInputLayer()
{
	if(m_currentLayer != NULL)
	{
		int selectedVariable = m_lbVariables->get_int_val();

		int typeIndex = m_lbLayerTypes->get_int_val();

		//ensure that the selected sublayer type is not 3D
		if(RenderableFactory::TypeInformation[typeIndex].type_metatype != RENDERABLE_3D)
		{
			Renderable2d *newLayer = (Renderable2d*)RenderableFactory::CreateRenderable(typeIndex);

			m_currentLayer->AssignLayerToVariable(selectedVariable, newLayer);
			m_currentLayer = newLayer;
			SetSelectedVariable();
			m_shouldUpdateLayers = true;
		}
	}
}

void LayerManager::event_NavigateInto(int value)
{
	m_Manager->NavigateInto();
}

void LayerManager::event_NavigateUp(int value)
{
	m_Manager->NavigateUp();
}

void LayerManager::NavigateInto()
{
	int selectedVariable = m_lbVariables->get_int_val();
	Renderable2d *sublayer = m_currentLayer->m_layerVariables[selectedVariable].GetRenderable();
	if((m_currentLayer != NULL) && (sublayer != NULL))
	{
		m_currentLayer = sublayer;

		SetGuiInformation();
		m_shouldUpdateLayers = true;
	}
}

void LayerManager::NavigateUp()
{
	if((m_currentLayer != NULL) && (m_currentLayer->ParentLayer != NULL))
	{
		m_currentLayer = m_currentLayer->ParentLayer;

		SetGuiInformation();
		m_shouldUpdateLayers = true;
	}
}

void LayerManager::event_AddLayer(int value)
{
	int typeIndex = m_Manager->m_lbLayerTypes->get_int_val();
	m_Manager->AddLayer(RenderableFactory::CreateRenderable(typeIndex));
}

void LayerManager::event_loadXML(int value)
{
	m_Manager->RemoveAllLayers();
	m_Manager->LoadFromXml();
	m_Manager->SetSelectedLayer();
}

void LayerManager::event_saveXML(int value)
{
	m_Manager->SaveToXml();
}

LayerManager::LayerManager(void)
{
	m_shouldUpdateLayers = false;
	m_currentLayer = NULL;

	m_Manager = this;

	m_width = 0;
	m_height = 0;

	m_renderableCount = 0;
}

LayerManager::~LayerManager(void)
{
	//dispose of layers
	RemoveAllLayers();
}

void LayerManager::RemoveAllLayers()
{
	m_currentLayer = NULL;

	for(int i = 0; i < m_renderableCount; i++)
		delete m_renderableList[i];

	m_renderableCount = 0;
}

void LayerManager::GetWorldCoordinates(float &x, float &y)
{
	m_mapping2d.GetWorldCoordinates(x, y);
}

void LayerManager::Translate(float delta_x, float delta_y)
{
	if(m_currentLayer != NULL)
	{
		m_currentLayer->Translate(delta_x * m_mapping2d.GetSpaceWidth(), delta_y * m_mapping2d.GetSpaceHeight());
		m_shouldUpdateLayers = true;
	}
}

void LayerManager::Scale(float scale_x, float scale_y)
{
	if(m_currentLayer != NULL)
	{
		m_currentLayer->Scale(scale_x, scale_y);
		m_shouldUpdateLayers = true;
	}
}

void LayerManager::IncrementParameter(float delta)
{
	if(m_currentLayer != NULL)
	{
		int selectedVariable = m_lbVariables->get_int_val();
		m_currentLayer->IncrementParameter(selectedVariable, delta);
		m_shouldUpdateLayers = true;
		SetVariableOverrideValue();
		UpdateTextures();

	}
}

//just for active layer
void LayerManager::UpdateTextures()
{
	if(m_currentLayer != NULL)
		m_currentLayer->Update();
}

//just for active layer
void LayerManager::Render()
{
	if(m_currentLayer != NULL)
		m_currentLayer->Render();
}


void LayerManager::AddLayer(BaseRenderable * newLayer)
{
	if(m_renderableCount >= MAX_RENDERABLE)
	{
		//error, no more room
		//return -1;
		return;
	}

#ifdef ENABLE_LAYER_LIST
	m_renderableList[m_renderableCount] = newLayer;

	char layername[200];
	sprintf_s(layername, "%s %d", RenderableFactory::TypeInformation[newLayer->m_typeID].type_name, newLayer->id);

	m_lbLayers->add_item(newLayer->id, layername);

	//select new layer
	m_lbLayers->set_int_val(newLayer->id);

	m_renderableCount++;

#else
	if(m_renderableCount > 0)
		delete m_renderableList[0];

	m_renderableList[0] = newLayer;
	m_currentLayer = newLayer;
	m_renderableCount = 1;
#endif

	SetSelectedLayer();
}

void LayerManager::RemoveLayer()
{
#ifdef ENABLE_LAYER_LIST
	//getting ID of layer
	int id = m_lbLayers->get_int_val();

	int index = GetIdIndex(id);

	//does specified index exist in list
	if(index >= 0)
	{
		delete m_renderableList[index];

		//move everything back
		for(int i = index; i < (m_renderableCount-1); i++)
			m_renderableList[i] = m_renderableList[i+1];

		m_renderableCount--;
		m_lbLayers->delete_item(id);
		if(m_renderableCount>0)
		{
			m_lbLayers->set_int_val(m_renderableList[0]->id);
		}

		SetSelectedLayer();
	}
#endif

	m_shouldUpdateLayers = true;
}

int LayerManager::GetIdIndex(int id)
{
	for(int i = 0; i < m_renderableCount; i++)
	{
		if(m_renderableList[i]->id == id)
			return i;
	}

	return -1;
}

bool LayerManager::ShouldUpdateLayers()
{
	if(m_shouldUpdateLayers)
	{
		m_shouldUpdateLayers = false;
		return true;
	}

	return false;
}


void LayerManager::SetSelectedLayer()
{
	if(m_renderableCount <= 0)
	{
		m_currentLayer = NULL;
	}
	else
	{
#ifdef ENABLE_LAYER_LIST
		int selectedIndex = GetIdIndex(m_lbLayers->get_int_val());
		m_currentLayer = m_renderableList[selectedIndex];
#else
		m_currentLayer = m_renderableList[0];
#endif
	}

	SetGuiInformation();

	m_shouldUpdateLayers = true;
}

//void LayerManager::RecalculateWindowByLookat(Vector3d cameraLookat)
//{
//	if((m_renderMode == DISPLAY_3D) && (m_currentLayer != NULL))
//	{
//		//((Renderable3d*) m_currentLayer)->RecalculateWindowByLookat(cameraLookat);
//	}
//
//	m_shouldUpdateLayers = true;
//}
//
//void LayerManager::RecalculateWindowByViewMagnitude(float magnitude)
//{
//	if((m_renderMode == DISPLAY_3D) && (m_currentLayer != NULL))
//	{
//		//((Renderable3d*) m_currentLayer)->RecalculateWindowByViewMagnitude(magnitude);
//	}
//
//	m_shouldUpdateLayers = true;
//}