#include "RenderableFactory.h"

#include "NoiseSlice.h"
#include "SquareOverlay.h"
#include "Circles.h"
#include "BaseRenderable.h"
#include "Boxes.h"

LayerTypeInfo RenderableFactory::TypeInformation[] = {
	{"Noise", RENDERABLE_2D, DISPLAY_2D},
	{"Squares", RENDERABLE_2D_DISCRETE, DISPLAY_2D},
	{"Circles", RENDERABLE_2D_DISCRETE, DISPLAY_2D},
	{"Boxes", RENDERABLE_3D, DISPLAY_3D}
};

int RenderableFactory::LayerTypeCount = sizeof(TypeInformation) / sizeof(LayerTypeInfo);

BaseRenderable* RenderableFactory::CreateRenderable(int renderable_type)
{
	if(renderable_type < 0 || renderable_type >= LayerTypeCount)
		return NULL;

	BaseRenderable *retval;

	switch(renderable_type)
	{
	case 0:
		retval = new NoiseSlice(renderable_type);
		break;
	case 1:
		retval = new SquareOverlay(renderable_type);
		break;
	case 2:
		retval = new Circles(renderable_type);
		break;
	case 3:
		retval = new Boxes(renderable_type);
		break;
	default:
		return NULL;
		break;
	}

	retval->InitializeBase();
	return retval;
}

RenderableFactory::RenderableFactory(void)
{
	//private constructor, nobody interesting should call this.
}

RenderableFactory::~RenderableFactory(void)
{
}